@@ -396,18 +396,6 @@ bool move_source_start_button(obs_properties_t *props, obs_property_t *property,
 	return false;
 }
 
-void move_source_start_hotkey(void *data, obs_hotkey_id id,
-			      obs_hotkey_t *hotkey, bool pressed)
-{
-	if (!pressed)
-		return;
-	struct move_source_info *move_source = data;
-	move_filter_start_hotkey(&move_source->move_filter);
-
-	UNUSED_PARAMETER(id);
-	UNUSED_PARAMETER(hotkey);
-}
-
 void move_source_stop(struct move_source_info *move_source)
 {
 	move_filter_stop(&move_source->move_filter);
@@ -587,23 +575,12 @@ void move_source_update(void *data, obs_data_t *settings)
 {
 	struct move_source_info *move_source = data;
 
-	obs_source_t *parent =
-		obs_filter_get_parent(move_source->move_filter.source);
 	const char *source_name = obs_data_get_string(settings, S_SOURCE);
 	if (!move_source->source_name ||
 	    strcmp(move_source->source_name, source_name) != 0) {
 		move_source_source_changed(move_source, source_name);
 	}
 	move_filter_update(&move_source->move_filter, settings);
-	if (parent && move_source->move_filter.filter_name &&
-	    move_source->move_filter.move_start_hotkey ==
-		    OBS_INVALID_HOTKEY_ID) {
-		move_source->move_filter.move_start_hotkey =
-			obs_hotkey_register_source(
-				parent, move_source->move_filter.filter_name,
-				move_source->move_filter.filter_name,
-				move_source_start_hotkey, data);
-	}
 
 	move_source->change_visibility =
 		obs_data_get_int(settings, S_CHANGE_VISIBILITY);
@@ -1476,14 +1453,13 @@ static obs_properties_t *move_source_properties(void *data)
 
 void move_source_defaults(obs_data_t *settings)
 {
+	move_filter_defaults(settings);
 	obs_data_set_default_bool(settings, S_CUSTOM_DURATION, true);
 	obs_data_set_default_bool(settings, S_TRANSFORM, true);
-	obs_data_set_default_int(settings, S_DURATION, 300);
 	obs_data_set_default_int(settings, S_EASING_MATCH, EASE_IN_OUT);
 	obs_data_set_default_int(settings, S_EASING_FUNCTION_MATCH,
 				 EASING_CUBIC);
 	obs_data_set_default_double(settings, S_CURVE_MATCH, 0.0);
-	obs_data_set_default_bool(settings, S_ENABLED_MATCH_MOVING, true);
 }
 
 void move_source_video_render(void *data, gs_effect_t *effect)
